/*
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Scanner.java
 * Copyright (C) 2010 University of Waikato, Hamilton, New Zealand
 */

package adams.parser.basetime;

import adams.core.DateFormat;
import adams.core.base.BaseTime;

import java_cup.runtime.SymbolFactory;
import java.io.*;
import java.util.*;

/**
 * A scanner for time expressions.
 *
 * @author FracPete (fracpete at waikato dot ac dot nz)
 * @version $Revision: 6572 $
 */
%%
%caseless
%cup
%public
%class Scanner
%{
  // Author: FracPete (fracpete at waikato dot ac dot nz)
  // Version: $Revision: 6572 $
  protected SymbolFactory sf;

  protected static DateFormat m_Format;
  static {
    m_Format = new DateFormat(BaseTime.FORMAT);
  }

  public Scanner(InputStream r, SymbolFactory sf){
    this(r);
    this.sf = sf;
  }
%}
%eofval{
    return sf.newSymbol("EOF",sym.EOF);
%eofval}

%%
// types
"SECOND" { return sf.newSymbol("Second", sym.TYPE, new Amount(Calendar.SECOND,    1)); }
"MINUTE" { return sf.newSymbol("Minute", sym.TYPE, new Amount(Calendar.MINUTE,    1)); }
"HOUR"   { return sf.newSymbol("Hour",   sym.TYPE, new Amount(Calendar.HOUR,      1)); }

// numbers
[+][0-9][0-9]* { return sf.newSymbol("Number", sym.NUMBER, new Integer(yytext().substring(1))); }
[-][0-9][0-9]* { return sf.newSymbol("Number", sym.NUMBER, new Integer(yytext())); }

// time
[0-9][0-9]:[0-9][0-9]:[0-9][0-9] { return sf.newSymbol("Date", sym.DATE_ACTUAL, m_Format.parse(yytext())); }
"-INF" { return sf.newSymbol("-INF", sym.DATE_ACTUAL, m_Format.parse(BaseTime.INF_PAST_DATE)); }
"+INF" { return sf.newSymbol("+INF", sym.DATE_ACTUAL, m_Format.parse(BaseTime.INF_FUTURE_DATE)); }
"NOW"  { return sf.newSymbol("Now",  sym.DATE_ACTUAL, new Date()); }
"START" { return sf.newSymbol("Start", sym.DATE_START,  m_Format.parse(BaseTime.INF_PAST_DATE)); }
"END"   { return sf.newSymbol("End",   sym.DATE_END,    m_Format.parse(BaseTime.INF_FUTURE_DATE)); }

// whitespaces
[ \r\n\t\f] { /* ignore white space. */ }

// catch all
. { System.err.println("Illegal character: " + yytext()); }
